# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class EntityGroupUpdateRequest(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.entity_group_types import EntityGroupUpdateRequest

    EntityGroupUpdateRequest(
        foreign_id="your-group-id",
        name="AcmeConglomerate",
        email_to_name="acmegroup",
    )
    """

    foreign_id: typing.Optional[str] = pydantic.Field(alias="foreignId", default=None)
    name: typing.Optional[str] = None
    email_to_name: typing.Optional[str] = pydantic.Field(alias="emailToName", default=None)
    metadata: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    Metadata key/value pairs to associate with this group. Will overwrite existing metadata.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from .custom_payment_method_schema_id import CustomPaymentMethodSchemaId
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from .plaid_link_request import PlaidLinkRequest
from .bank_account_check_options import BankAccountCheckOptions


class PaymentMethodUpdateRequest_Custom(UniversalBaseModel):
    """
    Update a payment method. for non custom payment methods.

    Examples
    --------
    from mercoa.payment_method_types import PaymentMethodUpdateRequest_BankAccount

    PaymentMethodUpdateRequest_BankAccount(
        default_source=True,
        default_destination=True,
    )
    """

    type: typing.Literal["custom"] = "custom"
    foreign_id: typing.Optional[str] = pydantic.Field(alias="foreignId", default=None)
    account_name: typing.Optional[str] = pydantic.Field(alias="accountName", default=None)
    account_number: typing.Optional[str] = pydantic.Field(alias="accountNumber", default=None)
    available_balance: typing.Optional[float] = pydantic.Field(alias="availableBalance", default=None)
    schema_id: typing.Optional[CustomPaymentMethodSchemaId] = pydantic.Field(alias="schemaId", default=None)
    data: typing.Optional[typing.Dict[str, str]] = None
    default_source: typing.Optional[bool] = pydantic.Field(alias="defaultSource", default=None)
    default_destination: typing.Optional[bool] = pydantic.Field(alias="defaultDestination", default=None)
    external_accounting_system_id: typing.Optional[str] = pydantic.Field(
        alias="externalAccountingSystemId", default=None
    )
    frozen: typing.Optional[bool] = None
    metadata: typing.Optional[typing.Dict[str, str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class PaymentMethodUpdateRequest_BankAccount(UniversalBaseModel):
    """
    Update a payment method. for non custom payment methods.

    Examples
    --------
    from mercoa.payment_method_types import PaymentMethodUpdateRequest_BankAccount

    PaymentMethodUpdateRequest_BankAccount(
        default_source=True,
        default_destination=True,
    )
    """

    type: typing.Literal["bankAccount"] = "bankAccount"
    account_name: typing.Optional[str] = pydantic.Field(alias="accountName", default=None)
    plaid: typing.Optional[PlaidLinkRequest] = None
    check_options: typing.Optional[BankAccountCheckOptions] = pydantic.Field(alias="checkOptions", default=None)
    default_source: typing.Optional[bool] = pydantic.Field(alias="defaultSource", default=None)
    default_destination: typing.Optional[bool] = pydantic.Field(alias="defaultDestination", default=None)
    external_accounting_system_id: typing.Optional[str] = pydantic.Field(
        alias="externalAccountingSystemId", default=None
    )
    frozen: typing.Optional[bool] = None
    metadata: typing.Optional[typing.Dict[str, str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class PaymentMethodUpdateRequest_Card(UniversalBaseModel):
    """
    Update a payment method. for non custom payment methods.

    Examples
    --------
    from mercoa.payment_method_types import PaymentMethodUpdateRequest_BankAccount

    PaymentMethodUpdateRequest_BankAccount(
        default_source=True,
        default_destination=True,
    )
    """

    type: typing.Literal["card"] = "card"
    default_source: typing.Optional[bool] = pydantic.Field(alias="defaultSource", default=None)
    default_destination: typing.Optional[bool] = pydantic.Field(alias="defaultDestination", default=None)
    external_accounting_system_id: typing.Optional[str] = pydantic.Field(
        alias="externalAccountingSystemId", default=None
    )
    frozen: typing.Optional[bool] = None
    metadata: typing.Optional[typing.Dict[str, str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class PaymentMethodUpdateRequest_Check(UniversalBaseModel):
    """
    Update a payment method. for non custom payment methods.

    Examples
    --------
    from mercoa.payment_method_types import PaymentMethodUpdateRequest_BankAccount

    PaymentMethodUpdateRequest_BankAccount(
        default_source=True,
        default_destination=True,
    )
    """

    type: typing.Literal["check"] = "check"
    default_source: typing.Optional[bool] = pydantic.Field(alias="defaultSource", default=None)
    default_destination: typing.Optional[bool] = pydantic.Field(alias="defaultDestination", default=None)
    external_accounting_system_id: typing.Optional[str] = pydantic.Field(
        alias="externalAccountingSystemId", default=None
    )
    frozen: typing.Optional[bool] = None
    metadata: typing.Optional[typing.Dict[str, str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class PaymentMethodUpdateRequest_OffPlatform(UniversalBaseModel):
    """
    Update a payment method. for non custom payment methods.

    Examples
    --------
    from mercoa.payment_method_types import PaymentMethodUpdateRequest_BankAccount

    PaymentMethodUpdateRequest_BankAccount(
        default_source=True,
        default_destination=True,
    )
    """

    type: typing.Literal["offPlatform"] = "offPlatform"
    default_source: typing.Optional[bool] = pydantic.Field(alias="defaultSource", default=None)
    default_destination: typing.Optional[bool] = pydantic.Field(alias="defaultDestination", default=None)
    external_accounting_system_id: typing.Optional[str] = pydantic.Field(
        alias="externalAccountingSystemId", default=None
    )
    frozen: typing.Optional[bool] = None
    metadata: typing.Optional[typing.Dict[str, str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class PaymentMethodUpdateRequest_Utility(UniversalBaseModel):
    """
    Update a payment method. for non custom payment methods.

    Examples
    --------
    from mercoa.payment_method_types import PaymentMethodUpdateRequest_BankAccount

    PaymentMethodUpdateRequest_BankAccount(
        default_source=True,
        default_destination=True,
    )
    """

    type: typing.Literal["utility"] = "utility"
    default_source: typing.Optional[bool] = pydantic.Field(alias="defaultSource", default=None)
    default_destination: typing.Optional[bool] = pydantic.Field(alias="defaultDestination", default=None)
    external_accounting_system_id: typing.Optional[str] = pydantic.Field(
        alias="externalAccountingSystemId", default=None
    )
    frozen: typing.Optional[bool] = None
    metadata: typing.Optional[typing.Dict[str, str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


"""
from mercoa.payment_method_types import PaymentMethodUpdateRequest_BankAccount

PaymentMethodUpdateRequest_BankAccount(
    default_source=True,
    default_destination=True,
)
"""
PaymentMethodUpdateRequest = typing.Union[
    PaymentMethodUpdateRequest_Custom,
    PaymentMethodUpdateRequest_BankAccount,
    PaymentMethodUpdateRequest_Card,
    PaymentMethodUpdateRequest_Check,
    PaymentMethodUpdateRequest_OffPlatform,
    PaymentMethodUpdateRequest_Utility,
]



# This file was auto-generated by Fern from our API Definition.

import typing

InvoiceLineItemCategory = typing.Union[
    typing.Literal["EXPENSE", "REVENUE", "TAX", "SHIPPING", "DISCOUNT", "OTHER"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

from .calculate_fees_request import CalculateFeesRequest
from .calculate_payment_timing_request import CalculatePaymentTimingRequest
from .calculate_payment_timing_response import CalculatePaymentTimingResponse
from .estimated_timing import EstimatedTiming
from .invoice_timing import InvoiceTiming

__all__ = [
    "CalculateFeesRequest",
    "CalculatePaymentTimingRequest",
    "CalculatePaymentTimingResponse",
    "EstimatedTiming",
    "InvoiceTiming",
]



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ...entity_types.types.entity_id import EntityId
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class EntityGroupUserEntityRequest(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.entity_group_types import EntityGroupUserEntityRequest

    EntityGroupUserEntityRequest(
        roles=["admin", "approver"],
        entity_id="ent_21661ac1-a2a8-4465-a6c0-64474ba8181d",
    )
    """

    roles: typing.List[str] = pydantic.Field()
    """
    List of roles. A role can be any string. For example: "payer", "approver", "viewer"
    """

    entity_id: EntityId = pydantic.Field(alias="entityId")
    """
    The IDs of the entities that these roles applies to.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .payment_method_base_request import PaymentMethodBaseRequest
import typing
import pydantic
from .plaid_link_request import PlaidLinkRequest
from .bank_account_check_options import BankAccountCheckOptions
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class BankAccountUpdateRequest(PaymentMethodBaseRequest):
    account_name: typing.Optional[str] = pydantic.Field(alias="accountName", default=None)
    plaid: typing.Optional[PlaidLinkRequest] = pydantic.Field(default=None)
    """
    If provided, will update a bank account using Plaid Link
    """

    check_options: typing.Optional[BankAccountCheckOptions] = pydantic.Field(alias="checkOptions", default=None)
    """
    If this bank account supports check printing, use this to enable check printing and set the check options. Checks will be printed directly from the bank account.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
from .invoice_id import InvoiceId
from .invoice_status import InvoiceStatus
import typing
import pydantic
from ...payment_method_types.types.currency_code import CurrencyCode
import datetime as dt
from ...entity_types.types.entity_id import EntityId
from ...entity_types.types.counterparty_response import CounterpartyResponse
from ...payment_method_types.types.payment_method_response import PaymentMethodResponse
from ...payment_method_types.types.payment_method_id import PaymentMethodId
from .payment_destination_options import PaymentDestinationOptions
from .comment_response import CommentResponse
from .invoice_line_item_response import InvoiceLineItemResponse
from .approval_slot import ApprovalSlot
from ...entity_types.types.approval_policy_response import ApprovalPolicyResponse
from ...entity_types.types.entity_user_response import EntityUserResponse
from .invoice_failure_type import InvoiceFailureType
from .invoice_fees_response import InvoiceFeesResponse
from .payment_schedule import PaymentSchedule
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class InvoiceResponse(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from mercoa.commons import Address, PhoneNumber
    from mercoa.entity_types import (
        ApprovalPolicyResponse,
        BusinessProfileResponse,
        CounterpartyCustomizationAccount,
        CounterpartyResponse,
        Ein,
        EntityUserResponse,
        IdentifierList_RolesList,
        ProfileResponse,
        Rule_Approver,
        TaxId,
        Trigger_Amount,
    )
    from mercoa.invoice_types import (
        ApprovalSlot,
        AssociatedApprovalAction,
        CommentResponse,
        InvoiceLineItemResponse,
        InvoiceResponse,
        PaymentDestinationOptions_Check,
    )
    from mercoa.payment_method_types import (
        PaymentMethodResponse_BankAccount,
        PaymentMethodResponse_Check,
    )

    InvoiceResponse(
        id="inv_26e7b5d3-a739-4b23-9ad9-6aaa085f47a9",
        status="PAID",
        amount=100.0,
        currency="USD",
        invoice_date=datetime.datetime.fromisoformat(
            "2021-01-01 00:00:00+00:00",
        ),
        deduction_date=datetime.datetime.fromisoformat(
            "2021-01-01 00:00:00+00:00",
        ),
        settlement_date=datetime.datetime.fromisoformat(
            "2021-01-03 00:00:00+00:00",
        ),
        due_date=datetime.datetime.fromisoformat(
            "2021-01-31 00:00:00+00:00",
        ),
        invoice_number="INV-123",
        note_to_self="For the month of January",
        service_start_date=datetime.datetime.fromisoformat(
            "2021-01-01 00:00:00+00:00",
        ),
        service_end_date=datetime.datetime.fromisoformat(
            "2021-01-31 00:00:00+00:00",
        ),
        payer_id="ent_8545a84e-a45f-41bf-bdf1-33b42a55812c",
        payer=CounterpartyResponse(
            id="ent_8545a84e-a45f-41bf-bdf1-33b42a55812c",
            foreign_id="MY-DB-ID-12345",
            name="Acme Inc.",
            email="customer@acme.com",
            accepted_tos=True,
            status="verified",
            is_customer=True,
            is_payor=True,
            is_payee=False,
            is_network_payor=False,
            is_network_payee=False,
            account_type="business",
            updated_at=datetime.datetime.fromisoformat(
                "2024-01-02 00:00:00+00:00",
            ),
            created_at=datetime.datetime.fromisoformat(
                "2024-01-01 00:00:00+00:00",
            ),
            profile=ProfileResponse(
                business=BusinessProfileResponse(
                    email="customer@acme.com",
                    legal_business_name="Acme Inc.",
                    business_type="llc",
                    phone=PhoneNumber(
                        country_code="1",
                        number="4155551234",
                    ),
                    address=Address(
                        address_line_1="123 Main St",
                        address_line_2="Unit 1",
                        city="San Francisco",
                        state_or_province="CA",
                        postal_code="94105",
                        country="US",
                    ),
                    tax_id_provided=True,
                    tax_id=TaxId(
                        ein=Ein(
                            number="12-3456789",
                        ),
                    ),
                    owners_provided=True,
                ),
            ),
            accounts=[
                CounterpartyCustomizationAccount(
                    account_id="85866843",
                    postal_code="94105",
                    name_on_account="John Doe",
                )
            ],
            payment_methods=[
                PaymentMethodResponse_BankAccount(
                    id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
                    account_name="My Checking Account",
                    bank_name="Chase",
                    routing_number="12345678",
                    account_number="99988767623",
                    account_type="CHECKING",
                    status="VERIFIED",
                    is_default_source=True,
                    is_default_destination=True,
                    supported_currencies=["USD"],
                    metadata={},
                    frozen=False,
                    created_at=datetime.datetime.fromisoformat(
                        "2021-01-01 00:00:00+00:00",
                    ),
                    updated_at=datetime.datetime.fromisoformat(
                        "2021-01-01 00:00:00+00:00",
                    ),
                )
            ],
            counterparty_type=["ENTITY"],
        ),
        payment_source=PaymentMethodResponse_BankAccount(
            id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
            account_name="My Checking Account",
            bank_name="Chase",
            routing_number="12345678",
            account_number="99988767623",
            account_type="CHECKING",
            status="VERIFIED",
            is_default_source=True,
            is_default_destination=True,
            supported_currencies=["USD"],
            metadata={},
            frozen=False,
            created_at=datetime.datetime.fromisoformat(
                "2021-01-01 00:00:00+00:00",
            ),
            updated_at=datetime.datetime.fromisoformat(
                "2021-01-01 00:00:00+00:00",
            ),
        ),
        payment_source_id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
        vendor_id="ent_21661ac1-a2a8-4465-a6c0-64474ba8181d",
        vendor=CounterpartyResponse(
            id="ent_21661ac1-a2a8-4465-a6c0-64474ba8181d",
            foreign_id="MY-DB-ID-90909",
            name="Big Box Store",
            email="vendor@bigboxstore.com",
            accepted_tos=False,
            status="unverified",
            is_customer=False,
            is_payor=False,
            is_payee=True,
            is_network_payor=False,
            is_network_payee=False,
            account_type="business",
            updated_at=datetime.datetime.fromisoformat(
                "2024-01-02 00:00:00+00:00",
            ),
            created_at=datetime.datetime.fromisoformat(
                "2024-01-01 00:00:00+00:00",
            ),
            profile=ProfileResponse(
                business=BusinessProfileResponse(
                    email="vendor@bigboxstore.com",
                    legal_business_name="Big Box Store",
                    business_type="publicCorporation",
                    tax_id_provided=False,
                    owners_provided=False,
                ),
            ),
            payment_methods=[
                PaymentMethodResponse_BankAccount(
                    id="pm_7610541f-4619-4033-8620-cfccfb811293",
                    account_name="Vendor Checking Account",
                    bank_name="Chase",
                    routing_number="66554433",
                    account_number="55934059697648",
                    account_type="CHECKING",
                    status="NEW",
                    is_default_source=True,
                    is_default_destination=True,
                    supported_currencies=["USD"],
                    metadata={},
                    frozen=False,
                    created_at=datetime.datetime.fromisoformat(
                        "2021-01-01 00:00:00+00:00",
                    ),
                    updated_at=datetime.datetime.fromisoformat(
                        "2021-01-01 00:00:00+00:00",
                    ),
                )
            ],
            counterparty_type=["ENTITY"],
        ),
        payment_destination=PaymentMethodResponse_Check(
            id="pm_5fde2f4a-facc-48ef-8f0d-6b7d087c7b18",
            pay_to_the_order_of="John Doe",
            address_line_1="123 Main St",
            address_line_2="Apt 1",
            city="New York",
            state_or_province="NY",
            postal_code="10001",
            country="US",
            is_default_source=False,
            is_default_destination=True,
            supported_currencies=["USD"],
            metadata={},
            frozen=False,
            created_at=datetime.datetime.fromisoformat(
                "2021-01-01 00:00:00+00:00",
            ),
            updated_at=datetime.datetime.fromisoformat(
                "2021-01-01 00:00:00+00:00",
            ),
        ),
        payment_destination_id="pm_5fde2f4a-facc-48ef-8f0d-6b7d087c7b18",
        payment_destination_options=PaymentDestinationOptions_Check(
            delivery="MAIL",
        ),
        payment_destination_confirmed=True,
        has_documents=True,
        has_source_email=True,
        comments=[
            CommentResponse(
                id="ic_b3525b66-da94-4525-9f31-426bcf657128",
                text="This is an approval comment",
                user=EntityUserResponse(
                    id="user_ec3aafc8-ea86-408a-a6c1-545497badbbb",
                    foreign_id="MY-DB-ID-12345",
                    email="john.doe@acme.com",
                    name="John Doe",
                    roles=["admin", "approver"],
                    created_at=datetime.datetime.fromisoformat(
                        "2024-01-01 00:00:00+00:00",
                    ),
                    updated_at=datetime.datetime.fromisoformat(
                        "2024-01-01 00:00:00+00:00",
                    ),
                ),
                associated_approval_action=AssociatedApprovalAction(
                    user_id="user_e24fc81c-c5ee-47e8-af42-4fe29d895506",
                    action="APPROVE",
                ),
                created_at=datetime.datetime.fromisoformat(
                    "2021-01-01 00:00:00+00:00",
                ),
                updated_at=datetime.datetime.fromisoformat(
                    "2021-01-01 00:00:00+00:00",
                ),
            )
        ],
        line_items=[
            InvoiceLineItemResponse(
                id="inli_26672f38-eb9a-48f1-a7a0-f1b855e38cd7",
                amount=100.0,
                currency="USD",
                description="Product A",
                name="Product A",
                quantity=1.0,
                unit_price=100.0,
                category="EXPENSE",
                service_start_date=datetime.datetime.fromisoformat(
                    "2021-01-01 00:00:00+00:00",
                ),
                service_end_date=datetime.datetime.fromisoformat(
                    "2021-01-31 00:00:00+00:00",
                ),
                metadata={"key1": "value1", "key2": "value2"},
                gl_account_id="600394",
                created_at=datetime.datetime.fromisoformat(
                    "2021-01-01 00:00:00+00:00",
                ),
                updated_at=datetime.datetime.fromisoformat(
                    "2021-01-01 00:00:00+00:00",
                ),
            )
        ],
        approvers=[
            ApprovalSlot(
                approval_policy_id="apvl_5ce50275-1789-42ea-bc60-bb7e6d03635c",
                approval_slot_id="inap_9bb311c9-7c15-4c9e-8148-63814e0abec6",
                assigned_user_id="user_e24fc81c-c5ee-47e8-af42-4fe29d895506",
                action="APPROVE",
                eligible_user_ids=["user_e24fc81c-c5ee-47e8-af42-4fe29d895506"],
                eligible_roles=["admin"],
                date=datetime.datetime.fromisoformat(
                    "2021-01-01 00:00:00+00:00",
                ),
            )
        ],
        approval_policy=[
            ApprovalPolicyResponse(
                id="apvl_8545a84e-a45f-41bf-bdf1-33b42a55812c",
                trigger=[
                    Trigger_Amount(
                        amount=100.0,
                        currency="USD",
                    )
                ],
                rule=Rule_Approver(
                    num_approvers=2,
                    identifier_list=IdentifierList_RolesList(
                        value=["Admin", "Controller"]
                    ),
                ),
                upstream_policy_id="root",
                updated_at=datetime.datetime.fromisoformat(
                    "2024-01-02 00:00:00+00:00",
                ),
                created_at=datetime.datetime.fromisoformat(
                    "2024-01-01 00:00:00+00:00",
                ),
            )
        ],
        metadata={"key1": "value1", "key2": "value2"},
        foreign_id="YOUR-DATABASE-ID",
        creator_user=EntityUserResponse(
            id="user_ec3aafc8-ea86-408a-a6c1-545497badbbb",
            foreign_id="MY-DB-ID-12345",
            email="john.doe@acme.com",
            name="John Doe",
            roles=["admin", "approver"],
            created_at=datetime.datetime.fromisoformat(
                "2024-01-01 00:00:00+00:00",
            ),
            updated_at=datetime.datetime.fromisoformat(
                "2024-01-01 00:00:00+00:00",
            ),
        ),
        processed_at=datetime.datetime.fromisoformat(
            "2021-01-01 00:00:00+00:00",
        ),
        created_at=datetime.datetime.fromisoformat(
            "2021-01-01 00:00:00+00:00",
        ),
        updated_at=datetime.datetime.fromisoformat(
            "2021-01-01 00:00:00+00:00",
        ),
    )
    """

    id: InvoiceId
    status: InvoiceStatus
    amount: typing.Optional[float] = pydantic.Field(default=None)
    """
    Total amount of invoice in major units
    """

    currency: typing.Optional[CurrencyCode] = pydantic.Field(default=None)
    """
    Currency code for the amount. Defaults to USD.
    """

    invoice_date: typing.Optional[dt.datetime] = pydantic.Field(alias="invoiceDate", default=None)
    """
    Date the invoice was issued.
    """

    deduction_date: typing.Optional[dt.datetime] = pydantic.Field(alias="deductionDate", default=None)
    """
    Initial date when funds are scheduled to be deducted from payer's account. The actual deduction date may differ from this date, and will be reflected in the processedAt field.
    """

    next_deduction_date: typing.Optional[dt.datetime] = pydantic.Field(alias="nextDeductionDate", default=None)
    """
    If this is a recurring invoice, this will be the next date when funds are scheduled to be deducted from payer's account.
    """

    processed_at: typing.Optional[dt.datetime] = pydantic.Field(alias="processedAt", default=None)
    """
    Date when the invoice payment was processed.
    """

    settlement_date: typing.Optional[dt.datetime] = pydantic.Field(alias="settlementDate", default=None)
    """
    Date of funds settlement.
    """

    due_date: typing.Optional[dt.datetime] = pydantic.Field(alias="dueDate", default=None)
    """
    Due date of invoice.
    """

    invoice_number: typing.Optional[str] = pydantic.Field(alias="invoiceNumber", default=None)
    note_to_self: typing.Optional[str] = pydantic.Field(alias="noteToSelf", default=None)
    service_start_date: typing.Optional[dt.datetime] = pydantic.Field(alias="serviceStartDate", default=None)
    service_end_date: typing.Optional[dt.datetime] = pydantic.Field(alias="serviceEndDate", default=None)
    payer_id: typing.Optional[EntityId] = pydantic.Field(alias="payerId", default=None)
    payer: typing.Optional[CounterpartyResponse] = None
    payment_source: typing.Optional[PaymentMethodResponse] = pydantic.Field(alias="paymentSource", default=None)
    payment_source_id: typing.Optional[PaymentMethodId] = pydantic.Field(alias="paymentSourceId", default=None)
    vendor_id: typing.Optional[EntityId] = pydantic.Field(alias="vendorId", default=None)
    vendor: typing.Optional[CounterpartyResponse] = None
    payment_destination: typing.Optional[PaymentMethodResponse] = pydantic.Field(
        alias="paymentDestination", default=None
    )
    payment_destination_id: typing.Optional[PaymentMethodId] = pydantic.Field(
        alias="paymentDestinationId", default=None
    )
    payment_destination_options: typing.Optional[PaymentDestinationOptions] = pydantic.Field(
        alias="paymentDestinationOptions", default=None
    )
    payment_destination_confirmed: bool = pydantic.Field(alias="paymentDestinationConfirmed")
    """
    True if the payment destination has been confirmed by the vendor. False if the payment destination has been set (for example, a check to an address) but has not been confirmed by the vendor.
    """

    batch_payment: typing.Optional[bool] = pydantic.Field(alias="batchPayment", default=None)
    """
    If true, this invoice will be paid as a batch payment. Batches are automatically determined by Mercoa based on the payment source, destination, and scheduled payment date.
    """

    has_documents: bool = pydantic.Field(alias="hasDocuments")
    """
    True if the invoice has documents attached.
    """

    has_source_email: bool = pydantic.Field(alias="hasSourceEmail")
    """
    True if the invoice was created by an incoming email.
    """

    comments: typing.Optional[typing.List[CommentResponse]] = None
    line_items: typing.Optional[typing.List[InvoiceLineItemResponse]] = pydantic.Field(alias="lineItems", default=None)
    approvers: typing.List[ApprovalSlot]
    approval_policy: typing.List[ApprovalPolicyResponse] = pydantic.Field(alias="approvalPolicy")
    metadata: typing.Dict[str, str] = pydantic.Field()
    """
    Metadata associated with this invoice.
    """

    foreign_id: typing.Optional[str] = pydantic.Field(alias="foreignId", default=None)
    """
    The ID used to identify this invoice in your system. This ID must be unique within each creatorEntity in your system, e.g. two invoices with the same creatorEntity may not have the same foreign ID.
    """

    creator_entity_id: typing.Optional[EntityId] = pydantic.Field(alias="creatorEntityId", default=None)
    """
    The ID of the entity who created this invoice.
    """

    creator_user: typing.Optional[EntityUserResponse] = pydantic.Field(alias="creatorUser", default=None)
    """
    Entity user who created this invoice.
    """

    failure_type: typing.Optional[InvoiceFailureType] = pydantic.Field(alias="failureType", default=None)
    """
    If the invoice failed to be paid, this field will be populated with the type of failure.
    """

    created_at: dt.datetime = pydantic.Field(alias="createdAt")
    updated_at: dt.datetime = pydantic.Field(alias="updatedAt")
    fees: typing.Optional[InvoiceFeesResponse] = pydantic.Field(default=None)
    """
    Fees associated with this invoice.
    """

    payment_schedule: typing.Optional[PaymentSchedule] = pydantic.Field(alias="paymentSchedule", default=None)
    """
    If this is a recurring invoice, this will be the payment schedule for the invoice. If not provided, this will be a one-time invoice.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .payment_method_base_response import PaymentMethodBaseResponse
import pydantic
from .bank_type import BankType
from .bank_status import BankStatus
import typing
from .bank_account_check_options import BankAccountCheckOptions
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class BankAccountResponse(PaymentMethodBaseResponse):
    """
    Examples
    --------
    import datetime

    from mercoa.payment_method_types import BankAccountResponse

    BankAccountResponse(
        id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
        account_name="My Checking Account",
        bank_name="Chase",
        routing_number="12345678",
        account_number="99988767623",
        account_type="CHECKING",
        status="VERIFIED",
        is_default_source=True,
        is_default_destination=True,
        supported_currencies=["USD"],
        metadata={},
        frozen=False,
        created_at=datetime.datetime.fromisoformat(
            "2021-01-01 00:00:00+00:00",
        ),
        updated_at=datetime.datetime.fromisoformat(
            "2021-01-01 00:00:00+00:00",
        ),
    )
    """

    account_name: str = pydantic.Field(alias="accountName")
    bank_name: str = pydantic.Field(alias="bankName")
    routing_number: str = pydantic.Field(alias="routingNumber")
    account_number: str = pydantic.Field(alias="accountNumber")
    account_type: BankType = pydantic.Field(alias="accountType")
    status: BankStatus
    check_options: typing.Optional[BankAccountCheckOptions] = pydantic.Field(alias="checkOptions", default=None)
    """
    If check printing is enabled for the account, will return the check options for this bank account
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from .bank_type import BankType
from .plaid_link_request import PlaidLinkRequest
from .bank_account_check_options import BankAccountCheckOptions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from .card_type import CardType
from .card_brand import CardBrand
from .custom_payment_method_schema_id import CustomPaymentMethodSchemaId


class PaymentMethodRequest_BankAccount(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.payment_method_types import PaymentMethodRequest_BankAccount

    PaymentMethodRequest_BankAccount(
        routing_number="12345678",
        account_number="99988767623",
        account_type="CHECKING",
    )
    """

    type: typing.Literal["bankAccount"] = "bankAccount"
    account_name: typing.Optional[str] = pydantic.Field(alias="accountName", default=None)
    bank_name: typing.Optional[str] = pydantic.Field(alias="bankName", default=None)
    routing_number: str = pydantic.Field(alias="routingNumber")
    account_number: str = pydantic.Field(alias="accountNumber")
    account_type: BankType = pydantic.Field(alias="accountType")
    plaid: typing.Optional[PlaidLinkRequest] = None
    check_options: typing.Optional[BankAccountCheckOptions] = pydantic.Field(alias="checkOptions", default=None)
    default_source: typing.Optional[bool] = pydantic.Field(alias="defaultSource", default=None)
    default_destination: typing.Optional[bool] = pydantic.Field(alias="defaultDestination", default=None)
    external_accounting_system_id: typing.Optional[str] = pydantic.Field(
        alias="externalAccountingSystemId", default=None
    )
    frozen: typing.Optional[bool] = None
    metadata: typing.Optional[typing.Dict[str, str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class PaymentMethodRequest_Card(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.payment_method_types import PaymentMethodRequest_BankAccount

    PaymentMethodRequest_BankAccount(
        routing_number="12345678",
        account_number="99988767623",
        account_type="CHECKING",
    )
    """

    type: typing.Literal["card"] = "card"
    card_type: CardType = pydantic.Field(alias="cardType")
    card_brand: CardBrand = pydantic.Field(alias="cardBrand")
    last_four: str = pydantic.Field(alias="lastFour")
    exp_month: str = pydantic.Field(alias="expMonth")
    exp_year: str = pydantic.Field(alias="expYear")
    token: str
    default_source: typing.Optional[bool] = pydantic.Field(alias="defaultSource", default=None)
    default_destination: typing.Optional[bool] = pydantic.Field(alias="defaultDestination", default=None)
    external_accounting_system_id: typing.Optional[str] = pydantic.Field(
        alias="externalAccountingSystemId", default=None
    )
    frozen: typing.Optional[bool] = None
    metadata: typing.Optional[typing.Dict[str, str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class PaymentMethodRequest_Check(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.payment_method_types import PaymentMethodRequest_BankAccount

    PaymentMethodRequest_BankAccount(
        routing_number="12345678",
        account_number="99988767623",
        account_type="CHECKING",
    )
    """

    type: typing.Literal["check"] = "check"
    pay_to_the_order_of: str = pydantic.Field(alias="payToTheOrderOf")
    address_line_1: str = pydantic.Field(alias="addressLine1")
    address_line_2: typing.Optional[str] = pydantic.Field(alias="addressLine2", default=None)
    city: str
    state_or_province: str = pydantic.Field(alias="stateOrProvince")
    postal_code: str = pydantic.Field(alias="postalCode")
    country: str
    default_source: typing.Optional[bool] = pydantic.Field(alias="defaultSource", default=None)
    default_destination: typing.Optional[bool] = pydantic.Field(alias="defaultDestination", default=None)
    external_accounting_system_id: typing.Optional[str] = pydantic.Field(
        alias="externalAccountingSystemId", default=None
    )
    frozen: typing.Optional[bool] = None
    metadata: typing.Optional[typing.Dict[str, str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class PaymentMethodRequest_Custom(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.payment_method_types import PaymentMethodRequest_BankAccount

    PaymentMethodRequest_BankAccount(
        routing_number="12345678",
        account_number="99988767623",
        account_type="CHECKING",
    )
    """

    type: typing.Literal["custom"] = "custom"
    foreign_id: typing.Optional[str] = pydantic.Field(alias="foreignId", default=None)
    account_name: typing.Optional[str] = pydantic.Field(alias="accountName", default=None)
    account_number: typing.Optional[str] = pydantic.Field(alias="accountNumber", default=None)
    available_balance: typing.Optional[float] = pydantic.Field(alias="availableBalance", default=None)
    schema_id: CustomPaymentMethodSchemaId = pydantic.Field(alias="schemaId")
    data: typing.Dict[str, str]
    default_source: typing.Optional[bool] = pydantic.Field(alias="defaultSource", default=None)
    default_destination: typing.Optional[bool] = pydantic.Field(alias="defaultDestination", default=None)
    external_accounting_system_id: typing.Optional[str] = pydantic.Field(
        alias="externalAccountingSystemId", default=None
    )
    frozen: typing.Optional[bool] = None
    metadata: typing.Optional[typing.Dict[str, str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class PaymentMethodRequest_OffPlatform(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.payment_method_types import PaymentMethodRequest_BankAccount

    PaymentMethodRequest_BankAccount(
        routing_number="12345678",
        account_number="99988767623",
        account_type="CHECKING",
    )
    """

    type: typing.Literal["offPlatform"] = "offPlatform"
    default_source: typing.Optional[bool] = pydantic.Field(alias="defaultSource", default=None)
    default_destination: typing.Optional[bool] = pydantic.Field(alias="defaultDestination", default=None)
    external_accounting_system_id: typing.Optional[str] = pydantic.Field(
        alias="externalAccountingSystemId", default=None
    )
    frozen: typing.Optional[bool] = None
    metadata: typing.Optional[typing.Dict[str, str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class PaymentMethodRequest_Utility(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.payment_method_types import PaymentMethodRequest_BankAccount

    PaymentMethodRequest_BankAccount(
        routing_number="12345678",
        account_number="99988767623",
        account_type="CHECKING",
    )
    """

    type: typing.Literal["utility"] = "utility"
    utility_id: str = pydantic.Field(alias="utilityId")
    default_source: typing.Optional[bool] = pydantic.Field(alias="defaultSource", default=None)
    default_destination: typing.Optional[bool] = pydantic.Field(alias="defaultDestination", default=None)
    external_accounting_system_id: typing.Optional[str] = pydantic.Field(
        alias="externalAccountingSystemId", default=None
    )
    frozen: typing.Optional[bool] = None
    metadata: typing.Optional[typing.Dict[str, str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


"""
from mercoa.payment_method_types import PaymentMethodRequest_BankAccount

PaymentMethodRequest_BankAccount(
    routing_number="12345678",
    account_number="99988767623",
    account_type="CHECKING",
)
"""
PaymentMethodRequest = typing.Union[
    PaymentMethodRequest_BankAccount,
    PaymentMethodRequest_Card,
    PaymentMethodRequest_Check,
    PaymentMethodRequest_Custom,
    PaymentMethodRequest_OffPlatform,
    PaymentMethodRequest_Utility,
]



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from .custom_payment_method_schema_field_type import CustomPaymentMethodSchemaFieldType
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class CustomPaymentMethodSchemaField(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.payment_method_types import CustomPaymentMethodSchemaField

    CustomPaymentMethodSchemaField(
        name="payToTheOrderOf",
        display_name="Pay To The Order Of",
        type="text",
        optional=False,
    )
    """

    name: str
    display_name: typing.Optional[str] = pydantic.Field(alias="displayName", default=None)
    type: CustomPaymentMethodSchemaFieldType
    optional: bool = pydantic.Field()
    """
    Indicates whether this field is optional
    """

    use_as_account_name: typing.Optional[bool] = pydantic.Field(alias="useAsAccountName", default=None)
    """
    Indicates whether this field should be used as the name of the payment method. Only one field can be used as the name. Will set the accountName field of the payment method to the value of this field.
    """

    use_as_account_number: typing.Optional[bool] = pydantic.Field(alias="useAsAccountNumber", default=None)
    """
    Indicates whether this field should be used as the account number of the payment method. Only one field can be used as the account number. Will set the accountNumber field of the payment method to the value of this field.
    """

    options: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    When type is 'select', provide options that can be selected
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
from ...entity_types.types.entity_id import EntityId
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class CounterpartyEventWebhook(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.webhooks import CounterpartyEventWebhook

    CounterpartyEventWebhook(
        event_type="counterparty.onboarding.completed",
        entity_id="ent_21661ac1-a2a8-4465-a6c0-64474ba8181d",
        counterparty_id="ent_8545a84e-a45f-41bf-bdf1-33b42a55812c",
    )
    """

    event_type: str = pydantic.Field(alias="eventType")
    entity_id: EntityId = pydantic.Field(alias="entityId")
    """
    The ID of the entity that owns the counterparty relationship
    """

    counterparty_id: EntityId = pydantic.Field(alias="counterpartyId")
    """
    The ID of the counterparty
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .payment_method_base_response import PaymentMethodBaseResponse
import typing
import pydantic
from .custom_payment_method_schema_id import CustomPaymentMethodSchemaId
from .custom_payment_method_schema_response import CustomPaymentMethodSchemaResponse
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class CustomPaymentMethodResponse(PaymentMethodBaseResponse):
    foreign_id: typing.Optional[str] = pydantic.Field(alias="foreignId", default=None)
    """
    ID for this payment method in your system
    """

    account_name: typing.Optional[str] = pydantic.Field(alias="accountName", default=None)
    account_number: typing.Optional[str] = pydantic.Field(alias="accountNumber", default=None)
    available_balance: typing.Optional[float] = pydantic.Field(alias="availableBalance", default=None)
    """
    The available balance for this payment method.
    """

    schema_id: CustomPaymentMethodSchemaId = pydantic.Field(alias="schemaId")
    """
    Payment method schema used for this payment method. Defines the fields that this payment method contains.
    """

    schema_: CustomPaymentMethodSchemaResponse = pydantic.Field(alias="schema")
    data: typing.Dict[str, str] = pydantic.Field()
    """
    Object of key/value pairs that matches the keys in the linked payment method schema.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .types import (
    EntityGroupAddEntitiesRequest,
    EntityGroupCreateRequest,
    EntityGroupFindResponse,
    EntityGroupId,
    EntityGroupRemoveEntitiesRequest,
    EntityGroupResponse,
    EntityGroupUpdateRequest,
    EntityGroupUserEntityRequest,
    EntityGroupUserEntityResponse,
    EntityGroupUserRequest,
    EntityGroupUserResponse,
    EntityIdOrBoolean,
    FindEntityGroupUserResponse,
)

__all__ = [
    "EntityGroupAddEntitiesRequest",
    "EntityGroupCreateRequest",
    "EntityGroupFindResponse",
    "EntityGroupId",
    "EntityGroupRemoveEntitiesRequest",
    "EntityGroupResponse",
    "EntityGroupUpdateRequest",
    "EntityGroupUserEntityRequest",
    "EntityGroupUserEntityResponse",
    "EntityGroupUserRequest",
    "EntityGroupUserResponse",
    "EntityIdOrBoolean",
    "FindEntityGroupUserResponse",
]



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...entity_types.types.entity_id import EntityId
from ...core.request_options import RequestOptions
from ...entity_types.types.representative_response import RepresentativeResponse
from ...core.jsonable_encoder import jsonable_encoder
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...core.pydantic_utilities import parse_obj_as
from ...commons.errors.bad_request import BadRequest
from ...commons.errors.unauthorized import Unauthorized
from ...commons.errors.forbidden import Forbidden
from ...commons.errors.not_found import NotFound
from ...commons.errors.conflict import Conflict
from ...commons.errors.internal_server_error import InternalServerError
from ...commons.errors.unimplemented import Unimplemented
from ...entity_types.types.representative_request import RepresentativeRequest
from ...entity_types.types.representative_id import RepresentativeId
from ...entity_types.types.representative_update_request import RepresentativeUpdateRequest
from ...core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RepresentativeClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_all(
        self, entity_id: EntityId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[RepresentativeResponse]:
        """
        Get representatives for an entity

        Parameters
        ----------
        entity_id : EntityId
            Entity ID or Entity ForeignID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[RepresentativeResponse]

        Examples
        --------
        from mercoa import Mercoa

        client = Mercoa(
            token="YOUR_TOKEN",
        )
        client.entity.representative.get_all(
            entity_id="ent_8545a84e-a45f-41bf-bdf1-33b42a55812c",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"entity/{jsonable_encoder(entity_id)}/representatives",
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                typing.List[RepresentativeResponse],
                parse_obj_as(
                    type_=typing.List[RepresentativeResponse],  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        entity_id: EntityId,
        *,
        request: RepresentativeRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RepresentativeResponse:
        """
        Parameters
        ----------
        entity_id : EntityId
            Entity ID or Entity ForeignID

        request : RepresentativeRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RepresentativeResponse

        Examples
        --------
        from mercoa import Mercoa
        from mercoa.commons import (
            Address,
            BirthDate,
            FullName,
            IndividualGovernmentId,
            PhoneNumber,
        )
        from mercoa.entity_types import RepresentativeRequest, Responsibilities

        client = Mercoa(
            token="YOUR_TOKEN",
        )
        client.entity.representative.create(
            entity_id="ent_8545a84e-a45f-41bf-bdf1-33b42a55812c",
            request=RepresentativeRequest(
                name=FullName(
                    first_name="John",
                    middle_name="Quincy",
                    last_name="Adams",
                    suffix="Jr.",
                ),
                phone=PhoneNumber(
                    country_code="1",
                    number="4155551234",
                ),
                email="john.doe@acme.com",
                address=Address(
                    address_line_1="123 Main St",
                    address_line_2="Unit 1",
                    city="San Francisco",
                    state_or_province="CA",
                    postal_code="94105",
                    country="US",
                ),
                birth_date=BirthDate(
                    day="1",
                    month="1",
                    year="1980",
                ),
                government_id=IndividualGovernmentId(
                    ssn="123-45-6789",
                ),
                responsibilities=Responsibilities(
                    is_owner=True,
                    ownership_percentage=40,
                    is_controller=True,
                ),
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"entity/{jsonable_encoder(entity_id)}/representative",
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                RepresentativeResponse,
                parse_obj_as(
                    type_=RepresentativeResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self,
        entity_id: EntityId,
        representative_id: RepresentativeId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RepresentativeResponse:
        """
        Parameters
        ----------
        entity_id : EntityId
            Entity ID or Entity ForeignID

        representative_id : RepresentativeId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RepresentativeResponse

        Examples
        --------
        from mercoa import Mercoa

        client = Mercoa(
            token="YOUR_TOKEN",
        )
        client.entity.representative.get(
            entity_id="ent_8545a84e-a45f-41bf-bdf1-33b42a55812c",
            representative_id="rep_7df2974a-4069-454c-912f-7e58ebe030fb",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"entity/{jsonable_encoder(entity_id)}/representative/{jsonable_encoder(representative_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                RepresentativeResponse,
                parse_obj_as(
                    type_=RepresentativeResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        entity_id: EntityId,
        representative_id: RepresentativeId,
        *,
        request: RepresentativeUpdateRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RepresentativeResponse:
        """
        Parameters
        ----------
        entity_id : EntityId
            Entity ID or Entity ForeignID

        representative_id : RepresentativeId

        request : RepresentativeUpdateRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RepresentativeResponse

        Examples
        --------
        from mercoa import Mercoa
        from mercoa.commons import (
            Address,
            BirthDate,
            FullName,
            IndividualGovernmentId,
            PhoneNumber,
        )
        from mercoa.entity_types import RepresentativeUpdateRequest, Responsibilities

        client = Mercoa(
            token="YOUR_TOKEN",
        )
        client.entity.representative.update(
            entity_id="ent_8545a84e-a45f-41bf-bdf1-33b42a55812c",
            representative_id="rep_7df2974a-4069-454c-912f-7e58ebe030fb",
            request=RepresentativeUpdateRequest(
                name=FullName(
                    first_name="John",
                    middle_name="Quincy",
                    last_name="Adams",
                    suffix="Jr.",
                ),
                phone=PhoneNumber(
                    country_code="1",
                    number="4155551234",
                ),
                email="john.doe@acme.com",
                address=Address(
                    address_line_1="123 Main St",
                    address_line_2="Unit 1",
                    city="San Francisco",
                    state_or_province="CA",
                    postal_code="94105",
                    country="US",
                ),
                birth_date=BirthDate(
                    day="1",
                    month="1",
                    year="1980",
                ),
                government_id=IndividualGovernmentId(
                    ssn="123-45-6789",
                ),
                responsibilities=Responsibilities(
                    is_owner=True,
                    ownership_percentage=40,
                    is_controller=True,
                ),
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"entity/{jsonable_encoder(entity_id)}/representative/{jsonable_encoder(representative_id)}",
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                RepresentativeResponse,
                parse_obj_as(
                    type_=RepresentativeResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self,
        entity_id: EntityId,
        representative_id: RepresentativeId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Parameters
        ----------
        entity_id : EntityId
            Entity ID or Entity ForeignID

        representative_id : RepresentativeId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from mercoa import Mercoa

        client = Mercoa(
            token="YOUR_TOKEN",
        )
        client.entity.representative.delete(
            entity_id="ent_8545a84e-a45f-41bf-bdf1-33b42a55812c",
            representative_id="rep_7df2974a-4069-454c-912f-7e58ebe030fb",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"entity/{jsonable_encoder(entity_id)}/representative/{jsonable_encoder(representative_id)}",
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRepresentativeClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_all(
        self, entity_id: EntityId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[RepresentativeResponse]:
        """
        Get representatives for an entity

        Parameters
        ----------
        entity_id : EntityId
            Entity ID or Entity ForeignID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[RepresentativeResponse]

        Examples
        --------
        import asyncio

        from mercoa import AsyncMercoa

        client = AsyncMercoa(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.entity.representative.get_all(
                entity_id="ent_8545a84e-a45f-41bf-bdf1-33b42a55812c",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"entity/{jsonable_encoder(entity_id)}/representatives",
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                typing.List[RepresentativeResponse],
                parse_obj_as(
                    type_=typing.List[RepresentativeResponse],  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        entity_id: EntityId,
        *,
        request: RepresentativeRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RepresentativeResponse:
        """
        Parameters
        ----------
        entity_id : EntityId
            Entity ID or Entity ForeignID

        request : RepresentativeRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RepresentativeResponse

        Examples
        --------
        import asyncio

        from mercoa import AsyncMercoa
        from mercoa.commons import (
            Address,
            BirthDate,
            FullName,
            IndividualGovernmentId,
            PhoneNumber,
        )
        from mercoa.entity_types import RepresentativeRequest, Responsibilities

        client = AsyncMercoa(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.entity.representative.create(
                entity_id="ent_8545a84e-a45f-41bf-bdf1-33b42a55812c",
                request=RepresentativeRequest(
                    name=FullName(
                        first_name="John",
                        middle_name="Quincy",
                        last_name="Adams",
                        suffix="Jr.",
                    ),
                    phone=PhoneNumber(
                        country_code="1",
                        number="4155551234",
                    ),
                    email="john.doe@acme.com",
                    address=Address(
                        address_line_1="123 Main St",
                        address_line_2="Unit 1",
                        city="San Francisco",
                        state_or_province="CA",
                        postal_code="94105",
                        country="US",
                    ),
                    birth_date=BirthDate(
                        day="1",
                        month="1",
                        year="1980",
                    ),
                    government_id=IndividualGovernmentId(
                        ssn="123-45-6789",
                    ),
                    responsibilities=Responsibilities(
                        is_owner=True,
                        ownership_percentage=40,
                        is_controller=True,
                    ),
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"entity/{jsonable_encoder(entity_id)}/representative",
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                RepresentativeResponse,
                parse_obj_as(
                    type_=RepresentativeResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self,
        entity_id: EntityId,
        representative_id: RepresentativeId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RepresentativeResponse:
        """
        Parameters
        ----------
        entity_id : EntityId
            Entity ID or Entity ForeignID

        representative_id : RepresentativeId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RepresentativeResponse

        Examples
        --------
        import asyncio

        from mercoa import AsyncMercoa

        client = AsyncMercoa(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.entity.representative.get(
                entity_id="ent_8545a84e-a45f-41bf-bdf1-33b42a55812c",
                representative_id="rep_7df2974a-4069-454c-912f-7e58ebe030fb",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"entity/{jsonable_encoder(entity_id)}/representative/{jsonable_encoder(representative_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                RepresentativeResponse,
                parse_obj_as(
                    type_=RepresentativeResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        entity_id: EntityId,
        representative_id: RepresentativeId,
        *,
        request: RepresentativeUpdateRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RepresentativeResponse:
        """
        Parameters
        ----------
        entity_id : EntityId
            Entity ID or Entity ForeignID

        representative_id : RepresentativeId

        request : RepresentativeUpdateRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RepresentativeResponse

        Examples
        --------
        import asyncio

        from mercoa import AsyncMercoa
        from mercoa.commons import (
            Address,
            BirthDate,
            FullName,
            IndividualGovernmentId,
            PhoneNumber,
        )
        from mercoa.entity_types import RepresentativeUpdateRequest, Responsibilities

        client = AsyncMercoa(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.entity.representative.update(
                entity_id="ent_8545a84e-a45f-41bf-bdf1-33b42a55812c",
                representative_id="rep_7df2974a-4069-454c-912f-7e58ebe030fb",
                request=RepresentativeUpdateRequest(
                    name=FullName(
                        first_name="John",
                        middle_name="Quincy",
                        last_name="Adams",
                        suffix="Jr.",
                    ),
                    phone=PhoneNumber(
                        country_code="1",
                        number="4155551234",
                    ),
                    email="john.doe@acme.com",
                    address=Address(
                        address_line_1="123 Main St",
                        address_line_2="Unit 1",
                        city="San Francisco",
                        state_or_province="CA",
                        postal_code="94105",
                        country="US",
                    ),
                    birth_date=BirthDate(
                        day="1",
                        month="1",
                        year="1980",
                    ),
                    government_id=IndividualGovernmentId(
                        ssn="123-45-6789",
                    ),
                    responsibilities=Responsibilities(
                        is_owner=True,
                        ownership_percentage=40,
                        is_controller=True,
                    ),
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"entity/{jsonable_encoder(entity_id)}/representative/{jsonable_encoder(representative_id)}",
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                RepresentativeResponse,
                parse_obj_as(
                    type_=RepresentativeResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self,
        entity_id: EntityId,
        representative_id: RepresentativeId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Parameters
        ----------
        entity_id : EntityId
            Entity ID or Entity ForeignID

        representative_id : RepresentativeId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from mercoa import AsyncMercoa

        client = AsyncMercoa(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.entity.representative.delete(
                entity_id="ent_8545a84e-a45f-41bf-bdf1-33b42a55812c",
                representative_id="rep_7df2974a-4069-454c-912f-7e58ebe030fb",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"entity/{jsonable_encoder(entity_id)}/representative/{jsonable_encoder(representative_id)}",
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
from .payment_method_id import PaymentMethodId
import pydantic
import typing
from .currency_code import CurrencyCode
import datetime as dt
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class PaymentMethodBaseResponse(UniversalBaseModel):
    id: PaymentMethodId
    is_default_source: bool = pydantic.Field(alias="isDefaultSource")
    """
    Indicates whether this payment method is the default source for the entity
    """

    is_default_destination: bool = pydantic.Field(alias="isDefaultDestination")
    """
    Indicates whether this payment method is the default destination for the entity
    """

    supported_currencies: typing.List[CurrencyCode] = pydantic.Field(alias="supportedCurrencies")
    external_accounting_system_id: typing.Optional[str] = pydantic.Field(
        alias="externalAccountingSystemId", default=None
    )
    """
    ID for this payment method in the external accounting system (e.g Rutter or Codat)
    """

    frozen: bool = pydantic.Field()
    """
    Frozen payment methods cannot be used for payments, but will still be returned in API responses.
    """

    metadata: typing.Dict[str, str] = pydantic.Field()
    """
    Metadata associated with this payment method.
    """

    created_at: dt.datetime = pydantic.Field(alias="createdAt")
    updated_at: dt.datetime = pydantic.Field(alias="updatedAt")

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .payment_method_base_request import PaymentMethodBaseRequest
from .card_type import CardType
import pydantic
from .card_brand import CardBrand
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class CardRequest(PaymentMethodBaseRequest):
    card_type: CardType = pydantic.Field(alias="cardType")
    card_brand: CardBrand = pydantic.Field(alias="cardBrand")
    last_four: str = pydantic.Field(alias="lastFour")
    exp_month: str = pydantic.Field(alias="expMonth")
    exp_year: str = pydantic.Field(alias="expYear")
    token: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
from .payment_rail_response import PaymentRailResponse
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class PaymentMethodsResponse(UniversalBaseModel):
    payer_payments: typing.List[PaymentRailResponse] = pydantic.Field(alias="payerPayments")
    """
    List of payment methods that can be used to pay invoices.
    """

    backup_disbursements: typing.List[PaymentRailResponse] = pydantic.Field(alias="backupDisbursements")
    """
    List of payment methods that can be created by a payor to send disbursements.
    """

    vendor_disbursements: typing.List[PaymentRailResponse] = pydantic.Field(alias="vendorDisbursements")
    """
    List of payment methods that can be created by a payee to receive disbursements.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .add_approver_request import AddApproverRequest
from .approval_request import ApprovalRequest
from .approval_slot import ApprovalSlot
from .approval_slot_assignment import ApprovalSlotAssignment
from .approval_slot_id import ApprovalSlotId
from .approver_action import ApproverAction
from .associated_approval_action import AssociatedApprovalAction
from .bank_account_payment_destination_options import BankAccountPaymentDestinationOptions
from .bank_delivery_method import BankDeliveryMethod
from .check_delivery_method import CheckDeliveryMethod
from .check_payment_destination_options import CheckPaymentDestinationOptions
from .comment_id import CommentId
from .comment_request import CommentRequest
from .comment_response import CommentResponse
from .day_of_week import DayOfWeek
from .find_invoice_response import FindInvoiceResponse
from .invoice_creation_request import InvoiceCreationRequest
from .invoice_creation_with_entity_group_request import InvoiceCreationWithEntityGroupRequest
from .invoice_creation_with_entity_request import InvoiceCreationWithEntityRequest
from .invoice_date_filter import InvoiceDateFilter
from .invoice_event import InvoiceEvent
from .invoice_events_response import InvoiceEventsResponse
from .invoice_failure_type import InvoiceFailureType
from .invoice_fees_request import InvoiceFeesRequest
from .invoice_fees_response import InvoiceFeesResponse
from .invoice_id import InvoiceId
from .invoice_line_item_category import InvoiceLineItemCategory
from .invoice_line_item_creation_request import InvoiceLineItemCreationRequest
from .invoice_line_item_id import InvoiceLineItemId
from .invoice_line_item_individual_update_request import InvoiceLineItemIndividualUpdateRequest
from .invoice_line_item_request_base import InvoiceLineItemRequestBase
from .invoice_line_item_response import InvoiceLineItemResponse
from .invoice_line_item_update_request import InvoiceLineItemUpdateRequest
from .invoice_metrics_group_by import InvoiceMetricsGroupBy
from .invoice_metrics_per_date_frequency import InvoiceMetricsPerDateFrequency
from .invoice_metrics_per_date_group_by import InvoiceMetricsPerDateGroupBy
from .invoice_metrics_per_date_response import InvoiceMetricsPerDateResponse
from .invoice_metrics_response import InvoiceMetricsResponse
from .invoice_order_by_field import InvoiceOrderByField
from .invoice_request_base import InvoiceRequestBase
from .invoice_response import InvoiceResponse
from .invoice_status import InvoiceStatus
from .invoice_update_request import InvoiceUpdateRequest
from .metadata_filter import MetadataFilter
from .payment_destination_options import (
    PaymentDestinationOptions,
    PaymentDestinationOptions_BankAccount,
    PaymentDestinationOptions_Check,
    PaymentDestinationOptions_Utility,
)
from .payment_month_repeat_type import PaymentMonthRepeatType
from .payment_month_schedule import PaymentMonthSchedule
from .payment_schedule import (
    PaymentSchedule,
    PaymentSchedule_Daily,
    PaymentSchedule_Monthly,
    PaymentSchedule_OneTime,
    PaymentSchedule_Weekly,
    PaymentSchedule_Yearly,
)
from .payment_schedule_base import PaymentScheduleBase
from .payment_schedule_end_condition import PaymentScheduleEndCondition
from .payment_type import PaymentType
from .payment_week_schedule import PaymentWeekSchedule
from .payment_year_schedule import PaymentYearSchedule
from .utility_payment_destination_options import UtilityPaymentDestinationOptions

__all__ = [
    "AddApproverRequest",
    "ApprovalRequest",
    "ApprovalSlot",
    "ApprovalSlotAssignment",
    "ApprovalSlotId",
    "ApproverAction",
    "AssociatedApprovalAction",
    "BankAccountPaymentDestinationOptions",
    "BankDeliveryMethod",
    "CheckDeliveryMethod",
    "CheckPaymentDestinationOptions",
    "CommentId",
    "CommentRequest",
    "CommentResponse",
    "DayOfWeek",
    "FindInvoiceResponse",
    "InvoiceCreationRequest",
    "InvoiceCreationWithEntityGroupRequest",
    "InvoiceCreationWithEntityRequest",
    "InvoiceDateFilter",
    "InvoiceEvent",
    "InvoiceEventsResponse",
    "InvoiceFailureType",
    "InvoiceFeesRequest",
    "InvoiceFeesResponse",
    "InvoiceId",
    "InvoiceLineItemCategory",
    "InvoiceLineItemCreationRequest",
    "InvoiceLineItemId",
    "InvoiceLineItemIndividualUpdateRequest",
    "InvoiceLineItemRequestBase",
    "InvoiceLineItemResponse",
    "InvoiceLineItemUpdateRequest",
    "InvoiceMetricsGroupBy",
    "InvoiceMetricsPerDateFrequency",
    "InvoiceMetricsPerDateGroupBy",
    "InvoiceMetricsPerDateResponse",
    "InvoiceMetricsResponse",
    "InvoiceOrderByField",
    "InvoiceRequestBase",
    "InvoiceResponse",
    "InvoiceStatus",
    "InvoiceUpdateRequest",
    "MetadataFilter",
    "PaymentDestinationOptions",
    "PaymentDestinationOptions_BankAccount",
    "PaymentDestinationOptions_Check",
    "PaymentDestinationOptions_Utility",
    "PaymentMonthRepeatType",
    "PaymentMonthSchedule",
    "PaymentSchedule",
    "PaymentScheduleBase",
    "PaymentScheduleEndCondition",
    "PaymentSchedule_Daily",
    "PaymentSchedule_Monthly",
    "PaymentSchedule_OneTime",
    "PaymentSchedule_Weekly",
    "PaymentSchedule_Yearly",
    "PaymentType",
    "PaymentWeekSchedule",
    "PaymentYearSchedule",
    "UtilityPaymentDestinationOptions",
]



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class Address(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.commons import Address

    Address(
        address_line_1="123 Main St",
        address_line_2="Unit 1",
        city="San Francisco",
        state_or_province="CA",
        postal_code="94105",
        country="US",
    )
    """

    address_line_1: str = pydantic.Field(alias="addressLine1")
    address_line_2: typing.Optional[str] = pydantic.Field(alias="addressLine2", default=None)
    city: str
    state_or_province: str = pydantic.Field(alias="stateOrProvince")
    """
    State or province code. Must be in the format XX.
    """

    postal_code: str = pydantic.Field(alias="postalCode")
    """
    Postal code. Must be in the format XXXXX or XXXXX-XXXX.
    """

    country: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class PlaidAccessTokenRequest(UniversalBaseModel):
    account_id: str = pydantic.Field(alias="accountId")
    """
    Plaid account ID
    """

    access_token: str = pydantic.Field(alias="accessToken")
    """
    Plaid access token for the account. If you already have an access token for the account (for example, you have linked the account to your app already), use this instead of publicToken.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class BankAddress(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.bank_lookup import BankAddress

    BankAddress(
        address="123 Main St",
        city="Anytown",
        state="CA",
        postal_code="12345",
        postal_code_extension="6789",
    )
    """

    address: str
    city: str
    state: str
    postal_code: str = pydantic.Field(alias="postalCode")
    postal_code_extension: str = pydantic.Field(alias="postalCodeExtension")

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
from .approval_slot_id import ApprovalSlotId
import pydantic
from ...entity_types.types.entity_user_id import EntityUserId
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class AddApproverRequest(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.invoice_types import AddApproverRequest

    AddApproverRequest(
        approval_slot_id="inap_9bb311c9-7c15-4c9e-8148-63814e0abec6",
        user_id="user_e24fc81c-c5ee-47e8-af42-4fe29d895506",
    )
    """

    approval_slot_id: typing.Optional[ApprovalSlotId] = pydantic.Field(alias="approvalSlotId", default=None)
    """
    The identifier for the approval slot this user is assigned to.
    """

    user_id: EntityUserId = pydantic.Field(alias="userId")
    """
    The ID or the Foreign ID of the user to add to the approval slot.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
from .approval_policy_id import ApprovalPolicyId
import typing
from .trigger import Trigger
from .rule import Rule
import pydantic
import datetime as dt
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ApprovalPolicyResponse(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from mercoa.entity_types import (
        ApprovalPolicyResponse,
        IdentifierList_RolesList,
        Rule_Approver,
        Trigger_Amount,
    )

    ApprovalPolicyResponse(
        id="apvl_8545a84e-a45f-41bf-bdf1-33b42a55812c",
        trigger=[
            Trigger_Amount(
                amount=100.0,
                currency="USD",
            )
        ],
        rule=Rule_Approver(
            num_approvers=2,
            identifier_list=IdentifierList_RolesList(value=["Admin", "Controller"]),
        ),
        upstream_policy_id="root",
        updated_at=datetime.datetime.fromisoformat(
            "2024-01-02 00:00:00+00:00",
        ),
        created_at=datetime.datetime.fromisoformat(
            "2024-01-01 00:00:00+00:00",
        ),
    )
    """

    id: ApprovalPolicyId
    trigger: typing.List[Trigger]
    rule: Rule
    upstream_policy_id: ApprovalPolicyId = pydantic.Field(alias="upstreamPolicyId")
    created_at: dt.datetime = pydantic.Field(alias="createdAt")
    updated_at: dt.datetime = pydantic.Field(alias="updatedAt")

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .types.calculate_fees_request import CalculateFeesRequest
from ..core.request_options import RequestOptions
from ..invoice_types.types.invoice_fees_response import InvoiceFeesResponse
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.pydantic_utilities import parse_obj_as
from ..commons.errors.bad_request import BadRequest
from ..commons.errors.unauthorized import Unauthorized
from ..commons.errors.forbidden import Forbidden
from ..commons.errors.not_found import NotFound
from ..commons.errors.conflict import Conflict
from ..commons.errors.internal_server_error import InternalServerError
from ..commons.errors.unimplemented import Unimplemented
from .types.calculate_payment_timing_request import CalculatePaymentTimingRequest
from .types.calculate_payment_timing_response import CalculatePaymentTimingResponse
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CalculateClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def fee(
        self, *, request: CalculateFeesRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> InvoiceFeesResponse:
        """
        Calculate the estimated fees associated with an payment given the amount, payment source, and disbursement method. Can be used to calculate fees for a payment before creating an invoice.

        Parameters
        ----------
        request : CalculateFeesRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InvoiceFeesResponse

        Examples
        --------
        from mercoa import Mercoa
        from mercoa.calculate import CalculateFeesRequest

        client = Mercoa(
            token="YOUR_TOKEN",
        )
        client.calculate.fee(
            request=CalculateFeesRequest(
                amount=100.0,
                payment_source_id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
                payment_destination_id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "fees",
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                InvoiceFeesResponse,
                parse_obj_as(
                    type_=InvoiceFeesResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def payment_timing(
        self, *, request: CalculatePaymentTimingRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> CalculatePaymentTimingResponse:
        """
        Calculate the estimated payment timing given the deduction date, payment source, and disbursement method. Can be used to calculate timing for a payment.

        Parameters
        ----------
        request : CalculatePaymentTimingRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CalculatePaymentTimingResponse

        Examples
        --------
        import datetime

        from mercoa import Mercoa
        from mercoa.calculate import EstimatedTiming

        client = Mercoa(
            token="YOUR_TOKEN",
        )
        client.calculate.payment_timing(
            request=EstimatedTiming(
                estimated_deduction_date=datetime.datetime.fromisoformat(
                    "2024-01-02 00:00:00+00:00",
                ),
                payment_source_id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
                payment_destination_id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "paymentTiming",
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                CalculatePaymentTimingResponse,
                parse_obj_as(
                    type_=CalculatePaymentTimingResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncCalculateClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def fee(
        self, *, request: CalculateFeesRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> InvoiceFeesResponse:
        """
        Calculate the estimated fees associated with an payment given the amount, payment source, and disbursement method. Can be used to calculate fees for a payment before creating an invoice.

        Parameters
        ----------
        request : CalculateFeesRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InvoiceFeesResponse

        Examples
        --------
        import asyncio

        from mercoa import AsyncMercoa
        from mercoa.calculate import CalculateFeesRequest

        client = AsyncMercoa(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.calculate.fee(
                request=CalculateFeesRequest(
                    amount=100.0,
                    payment_source_id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
                    payment_destination_id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fees",
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                InvoiceFeesResponse,
                parse_obj_as(
                    type_=InvoiceFeesResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def payment_timing(
        self, *, request: CalculatePaymentTimingRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> CalculatePaymentTimingResponse:
        """
        Calculate the estimated payment timing given the deduction date, payment source, and disbursement method. Can be used to calculate timing for a payment.

        Parameters
        ----------
        request : CalculatePaymentTimingRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CalculatePaymentTimingResponse

        Examples
        --------
        import asyncio
        import datetime

        from mercoa import AsyncMercoa
        from mercoa.calculate import EstimatedTiming

        client = AsyncMercoa(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.calculate.payment_timing(
                request=EstimatedTiming(
                    estimated_deduction_date=datetime.datetime.fromisoformat(
                        "2024-01-02 00:00:00+00:00",
                    ),
                    payment_source_id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
                    payment_destination_id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "paymentTiming",
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                CalculatePaymentTimingResponse,
                parse_obj_as(
                    type_=CalculatePaymentTimingResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from .counterparty_response import CounterpartyResponse
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class FindCounterpartiesResponse(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from mercoa.entity_types import (
        BusinessProfileResponse,
        CounterpartyResponse,
        FindCounterpartiesResponse,
        ProfileResponse,
    )
    from mercoa.payment_method_types import PaymentMethodResponse_BankAccount

    FindCounterpartiesResponse(
        count=1,
        has_more=False,
        data=[
            CounterpartyResponse(
                id="ent_21661ac1-a2a8-4465-a6c0-64474ba8181d",
                foreign_id="MY-DB-ID-90909",
                name="Big Box Store",
                email="vendor@bigboxstore.com",
                accepted_tos=False,
                status="unverified",
                is_customer=False,
                is_payor=False,
                is_payee=True,
                is_network_payor=False,
                is_network_payee=False,
                account_type="business",
                updated_at=datetime.datetime.fromisoformat(
                    "2024-01-02 00:00:00+00:00",
                ),
                created_at=datetime.datetime.fromisoformat(
                    "2024-01-01 00:00:00+00:00",
                ),
                profile=ProfileResponse(
                    business=BusinessProfileResponse(
                        email="vendor@bigboxstore.com",
                        legal_business_name="Big Box Store",
                        business_type="publicCorporation",
                        tax_id_provided=False,
                        owners_provided=False,
                    ),
                ),
                payment_methods=[
                    PaymentMethodResponse_BankAccount(
                        id="pm_7610541f-4619-4033-8620-cfccfb811293",
                        account_name="Vendor Checking Account",
                        bank_name="Chase",
                        routing_number="66554433",
                        account_number="55934059697648",
                        account_type="CHECKING",
                        status="NEW",
                        is_default_source=True,
                        is_default_destination=True,
                        supported_currencies=["USD"],
                        metadata={},
                        frozen=False,
                        created_at=datetime.datetime.fromisoformat(
                            "2021-01-01 00:00:00+00:00",
                        ),
                        updated_at=datetime.datetime.fromisoformat(
                            "2021-01-01 00:00:00+00:00",
                        ),
                    )
                ],
                counterparty_type=["ENTITY"],
            )
        ],
    )
    """

    count: int = pydantic.Field()
    """
    Total number of counterparties for the given filters. This value is not limited by the limit parameter. It is provided so that you can determine how many pages of results are available.
    """

    has_more: bool = pydantic.Field(alias="hasMore")
    """
    True if there are more counterparties available for the given filters.
    """

    data: typing.List[CounterpartyResponse]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from . import approval, comment, document, line_item, payment_links

__all__ = ["approval", "comment", "document", "line_item", "payment_links"]



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class FullName(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.commons import FullName

    FullName(
        first_name="John",
        middle_name="Quincy",
        last_name="Adams",
        suffix="Jr.",
    )
    """

    first_name: str = pydantic.Field(alias="firstName")
    middle_name: typing.Optional[str] = pydantic.Field(alias="middleName", default=None)
    last_name: str = pydantic.Field(alias="lastName")
    suffix: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from ...entity_types.types.entity_id import EntityId
from ...entity_types.types.entity_user_response import EntityUserResponse
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class CounterpartyWebhook(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from mercoa.entity_types import EntityUserResponse
    from mercoa.webhooks import CounterpartyWebhook

    CounterpartyWebhook(
        event_type="counterparty.added",
        payee_id=["ent_21661ac1-a2a8-4465-a6c0-64474ba8181d"],
        payor_id=["ent_8545a84e-a45f-41bf-bdf1-33b42a55812c"],
        user=EntityUserResponse(
            id="user_ec3aafc8-ea86-408a-a6c1-545497badbbb",
            foreign_id="MY-DB-ID-12345",
            email="john.doe@acme.com",
            name="John Doe",
            roles=["admin", "approver"],
            created_at=datetime.datetime.fromisoformat(
                "2024-01-01 00:00:00+00:00",
            ),
            updated_at=datetime.datetime.fromisoformat(
                "2024-01-01 00:00:00+00:00",
            ),
        ),
    )
    """

    event_type: str = pydantic.Field(alias="eventType")
    payee_id: typing.List[EntityId] = pydantic.Field(alias="payeeId")
    payor_id: typing.List[EntityId] = pydantic.Field(alias="payorId")
    user: typing.Optional[EntityUserResponse] = pydantic.Field(default=None)
    """
    User who initiated the change.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ...entity_types.types.entity_user_id import EntityUserId
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ApprovalRequest(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.invoice_types import ApprovalRequest

    ApprovalRequest(
        text="This is a reason for my action",
        user_id="user_e24fc81c-c5ee-47e8-af42-4fe29d895506",
    )
    """

    text: typing.Optional[str] = pydantic.Field(default=None)
    """
    Comment associated with this approval action.
    """

    user_id: EntityUserId = pydantic.Field(alias="userId")
    """
    The ID or the Foreign ID of the user
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
from .invoice_status import InvoiceStatus
import pydantic
from ...payment_method_types.types.currency_code import CurrencyCode
import datetime as dt
from ...entity_types.types.entity_id import EntityId
from ...payment_method_types.types.payment_method_id import PaymentMethodId
from .payment_destination_options import PaymentDestinationOptions
from .approval_slot_assignment import ApprovalSlotAssignment
from ...entity_types.types.entity_user_id import EntityUserId
from .invoice_failure_type import InvoiceFailureType
from .invoice_fees_request import InvoiceFeesRequest
from .payment_schedule import PaymentSchedule
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class InvoiceRequestBase(UniversalBaseModel):
    status: typing.Optional[InvoiceStatus] = None
    amount: typing.Optional[float] = pydantic.Field(default=None)
    """
    Total amount of invoice in major units. If the entered amount has more decimal places than the currency supports, trailing decimals will be truncated.
    """

    currency: typing.Optional[CurrencyCode] = pydantic.Field(default=None)
    """
    Currency code for the amount. Defaults to USD.
    """

    invoice_date: typing.Optional[dt.datetime] = pydantic.Field(alias="invoiceDate", default=None)
    """
    Date the invoice was issued.
    """

    deduction_date: typing.Optional[dt.datetime] = pydantic.Field(alias="deductionDate", default=None)
    """
    Initial date when funds are scheduled to be deducted from payer's account.
    """

    settlement_date: typing.Optional[dt.datetime] = pydantic.Field(alias="settlementDate", default=None)
    """
    Date of funds settlement.
    """

    due_date: typing.Optional[dt.datetime] = pydantic.Field(alias="dueDate", default=None)
    """
    Due date of invoice.
    """

    invoice_number: typing.Optional[str] = pydantic.Field(alias="invoiceNumber", default=None)
    note_to_self: typing.Optional[str] = pydantic.Field(alias="noteToSelf", default=None)
    """
    Note to self or memo on invoice.
    """

    service_start_date: typing.Optional[dt.datetime] = pydantic.Field(alias="serviceStartDate", default=None)
    service_end_date: typing.Optional[dt.datetime] = pydantic.Field(alias="serviceEndDate", default=None)
    payer_id: typing.Optional[EntityId] = pydantic.Field(alias="payerId", default=None)
    """
    ID or foreign ID of the payer of this invoice.
    """

    payment_source_id: typing.Optional[PaymentMethodId] = pydantic.Field(alias="paymentSourceId", default=None)
    """
    ID of payment source for this invoice. If not provided, will attempt to use the default payment source for the payer when creating an invoice if a default payment source exists for the payer.
    """

    vendor_id: typing.Optional[EntityId] = pydantic.Field(alias="vendorId", default=None)
    """
    ID or foreign ID of the vendor of this invoice.
    """

    payment_destination_id: typing.Optional[PaymentMethodId] = pydantic.Field(
        alias="paymentDestinationId", default=None
    )
    """
    ID of payment destination for this invoice. If not provided, will attempt to use the default payment destination for the vendor when creating an invoice if a default payment destination exists for the vendor.
    """

    payment_destination_options: typing.Optional[PaymentDestinationOptions] = pydantic.Field(
        alias="paymentDestinationOptions", default=None
    )
    """
    Options for the payment destination. Depending on the payment destination, this may include things such as check delivery method.
    """

    approvers: typing.Optional[typing.List[ApprovalSlotAssignment]] = pydantic.Field(default=None)
    """
    Set approvers for this invoice.
    """

    metadata: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    Metadata associated with this invoice.
    """

    foreign_id: typing.Optional[str] = pydantic.Field(alias="foreignId", default=None)
    """
    The ID used to identify this invoice in your system. This ID must be unique within each creatorEntity in your system, e.g. two invoices with the same creatorEntity may not have the same foreign ID.
    """

    document: typing.Optional[str] = pydantic.Field(default=None)
    """
    Base64 encoded image or PDF of invoice document. PNG, JPG, WEBP, and PDF are supported. 10MB max. If the invoice already has a document, this will add a new document to the invoice.
    """

    uploaded_image: typing.Optional[str] = pydantic.Field(alias="uploadedImage", default=None)
    """
    DEPRECATED. Use document field instead.
    """

    creator_user_id: typing.Optional[EntityUserId] = pydantic.Field(alias="creatorUserId", default=None)
    """
    User ID or Foreign ID of entity user who created this invoice.
    """

    failure_type: typing.Optional[InvoiceFailureType] = pydantic.Field(alias="failureType", default=None)
    """
    If the invoice failed to be paid, indicate the failure reason. Only applicable for invoices with custom payment methods.
    """

    fees: typing.Optional[InvoiceFeesRequest] = pydantic.Field(default=None)
    """
    If using a custom payment method, you can override the default fees for this invoice. If not provided, the default fees for the custom payment method will be used.
    """

    batch_payment: typing.Optional[bool] = pydantic.Field(alias="batchPayment", default=None)
    """
    If true, this invoice will be paid as a batch payment. Batches are automatically determined by Mercoa based on the payment source, destination, and scheduled payment date.
    """

    payment_schedule: typing.Optional[PaymentSchedule] = pydantic.Field(alias="paymentSchedule", default=None)
    """
    If this is a recurring invoice, this will be the payment schedule for the invoice. If not provided, this will be a one-time invoice.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.pydantic_utilities import UniversalBaseModel
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic
from .entity_user_id import EntityUserId


class IdentifierList_RolesList(UniversalBaseModel):
    value: typing.List[str]
    type: typing.Literal["rolesList"] = "rolesList"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


class IdentifierList_UserList(UniversalBaseModel):
    value: typing.List[EntityUserId]
    type: typing.Literal["userList"] = "userList"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


IdentifierList = typing.Union[IdentifierList_RolesList, IdentifierList_UserList]



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
from .email_sender_provider import EmailSenderProvider
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class EmailSenderResponse(UniversalBaseModel):
    provider: EmailSenderProvider
    from_email: str = pydantic.Field(alias="fromEmail")
    from_name: str = pydantic.Field(alias="fromName")
    has_api_key: bool = pydantic.Field(alias="hasApiKey")

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
from .entity_id import EntityId
import pydantic
from ...payment_method_types.types.payment_method_id import PaymentMethodId
from .acceleration_funds_balance_response import AccelerationFundsBalanceResponse
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class AccelerationFundsResponse(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.entity_types import (
        AccelerationFundsBalanceResponse,
        AccelerationFundsResponse,
    )

    AccelerationFundsResponse(
        entity_id="ent_8545a84e-a45f-41bf-bdf1-33b42a55812c",
        payment_method_id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
        available_balance=AccelerationFundsBalanceResponse(
            amount=100.0,
            currency="USD",
        ),
        pending_balance=AccelerationFundsBalanceResponse(
            amount=5.0,
            currency="USD",
        ),
    )
    """

    entity_id: EntityId = pydantic.Field(alias="entityId")
    """
    The ID of the entity that these acceleration funds belong to.
    """

    payment_method_id: PaymentMethodId = pydantic.Field(alias="paymentMethodId")
    """
    The ID of the payment method that the acceleration funds were accessed through.
    """

    available_balance: AccelerationFundsBalanceResponse = pydantic.Field(alias="availableBalance")
    pending_balance: AccelerationFundsBalanceResponse = pydantic.Field(alias="pendingBalance")

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
from .invoice_event import InvoiceEvent
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class InvoiceEventsResponse(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from mercoa.commons import Address, PhoneNumber
    from mercoa.entity_types import (
        BusinessProfileResponse,
        CounterpartyCustomizationAccount,
        CounterpartyResponse,
        Ein,
        ProfileResponse,
        TaxId,
    )
    from mercoa.invoice_types import (
        InvoiceEvent,
        InvoiceEventsResponse,
        InvoiceResponse,
    )
    from mercoa.payment_method_types import PaymentMethodResponse_BankAccount

    InvoiceEventsResponse(
        data=[
            InvoiceEvent(
                webhook_ids=["webhook_12345"],
                user_id="user_e24fc81c-c5ee-47e8-af42-4fe29d895506",
                data=InvoiceResponse(
                    id="inv_26e7b5d3-a739-4b23-9ad9-6aaa085f47a9",
                    status="DRAFT",
                    payer_id="ent_8545a84e-a45f-41bf-bdf1-33b42a55812c",
                    payer=CounterpartyResponse(
                        id="ent_8545a84e-a45f-41bf-bdf1-33b42a55812c",
                        foreign_id="MY-DB-ID-12345",
                        name="Acme Inc.",
                        email="customer@acme.com",
                        accepted_tos=True,
                        status="verified",
                        is_customer=True,
                        is_payor=True,
                        is_payee=False,
                        is_network_payor=False,
                        is_network_payee=False,
                        account_type="business",
                        updated_at=datetime.datetime.fromisoformat(
                            "2024-01-02 00:00:00+00:00",
                        ),
                        created_at=datetime.datetime.fromisoformat(
                            "2024-01-01 00:00:00+00:00",
                        ),
                        profile=ProfileResponse(
                            business=BusinessProfileResponse(
                                email="customer@acme.com",
                                legal_business_name="Acme Inc.",
                                business_type="llc",
                                phone=PhoneNumber(
                                    country_code="1",
                                    number="4155551234",
                                ),
                                address=Address(
                                    address_line_1="123 Main St",
                                    address_line_2="Unit 1",
                                    city="San Francisco",
                                    state_or_province="CA",
                                    postal_code="94105",
                                    country="US",
                                ),
                                tax_id_provided=True,
                                tax_id=TaxId(
                                    ein=Ein(
                                        number="12-3456789",
                                    ),
                                ),
                                owners_provided=True,
                            ),
                        ),
                        accounts=[
                            CounterpartyCustomizationAccount(
                                account_id="85866843",
                                postal_code="94105",
                                name_on_account="John Doe",
                            )
                        ],
                        payment_methods=[
                            PaymentMethodResponse_BankAccount(
                                id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
                                account_name="My Checking Account",
                                bank_name="Chase",
                                routing_number="12345678",
                                account_number="99988767623",
                                account_type="CHECKING",
                                status="VERIFIED",
                                is_default_source=True,
                                is_default_destination=True,
                                supported_currencies=["USD"],
                                metadata={},
                                frozen=False,
                                created_at=datetime.datetime.fromisoformat(
                                    "2021-01-01 00:00:00+00:00",
                                ),
                                updated_at=datetime.datetime.fromisoformat(
                                    "2021-01-01 00:00:00+00:00",
                                ),
                            )
                        ],
                        counterparty_type=["ENTITY"],
                    ),
                    payment_destination_confirmed=True,
                    has_documents=True,
                    has_source_email=True,
                    approvers=[],
                    approval_policy=[],
                    metadata={},
                    created_at=datetime.datetime.fromisoformat(
                        "2021-01-01 00:00:00+00:00",
                    ),
                    updated_at=datetime.datetime.fromisoformat(
                        "2021-01-01 00:00:00+00:00",
                    ),
                ),
                created_at=datetime.datetime.fromisoformat(
                    "2024-01-01 00:00:00+00:00",
                ),
            )
        ],
        count=1,
    )
    """

    data: typing.List[InvoiceEvent]
    count: int

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
from ...payment_method_types.types.payment_method_type import PaymentMethodType
import pydantic
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class PaymentRailResponse(UniversalBaseModel):
    type: PaymentMethodType
    name: str = pydantic.Field()
    """
    For custom payment methods, this is the ID of the schema.
    """

    active: bool
    available: typing.Optional[bool] = pydantic.Field(default=None)
    """
    unused
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
from ...commons.types.full_name import FullName
from ...commons.types.phone_number import PhoneNumber
from ...commons.types.address import Address
from ...commons.types.birth_date import BirthDate
import pydantic
from ...commons.types.individual_government_id import IndividualGovernmentId
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class IndividualProfileRequest(UniversalBaseModel):
    email: typing.Optional[str] = None
    name: FullName
    phone: typing.Optional[PhoneNumber] = None
    address: typing.Optional[Address] = None
    birth_date: typing.Optional[BirthDate] = pydantic.Field(alias="birthDate", default=None)
    government_id: typing.Optional[IndividualGovernmentId] = pydantic.Field(alias="governmentID", default=None)

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
from ...commons.types.string_or_string_array import StringOrStringArray
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class MetadataFilter(UniversalBaseModel):
    key: str
    value: StringOrStringArray = pydantic.Field()
    """
    If multiple values are provided, the filter will match if any of the values match (OR filter)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.pydantic_utilities import UniversalBaseModel
import typing
from .check_delivery_method import CheckDeliveryMethod
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic
from .bank_delivery_method import BankDeliveryMethod


class PaymentDestinationOptions_Check(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.invoice_types import PaymentDestinationOptions_Check

    PaymentDestinationOptions_Check(
        delivery="MAIL",
    )
    """

    type: typing.Literal["check"] = "check"
    delivery: typing.Optional[CheckDeliveryMethod] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class PaymentDestinationOptions_BankAccount(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.invoice_types import PaymentDestinationOptions_Check

    PaymentDestinationOptions_Check(
        delivery="MAIL",
    )
    """

    type: typing.Literal["bankAccount"] = "bankAccount"
    delivery: typing.Optional[BankDeliveryMethod] = None
    description: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class PaymentDestinationOptions_Utility(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.invoice_types import PaymentDestinationOptions_Check

    PaymentDestinationOptions_Check(
        delivery="MAIL",
    )
    """

    type: typing.Literal["utility"] = "utility"
    account_id: str = pydantic.Field(alias="accountId")

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


"""
from mercoa.invoice_types import PaymentDestinationOptions_Check

PaymentDestinationOptions_Check(
    delivery="MAIL",
)
"""
PaymentDestinationOptions = typing.Union[
    PaymentDestinationOptions_Check, PaymentDestinationOptions_BankAccount, PaymentDestinationOptions_Utility
]



# This file was auto-generated by Fern from our API Definition.

from .entity_group_user_entity_request import EntityGroupUserEntityRequest
from ...entity_types.types.entity_user_id import EntityUserId
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing
import pydantic


class EntityGroupUserEntityResponse(EntityGroupUserEntityRequest):
    """
    Examples
    --------
    from mercoa.entity_group_types import EntityGroupUserEntityResponse

    EntityGroupUserEntityResponse(
        id="user_ec3aafc8-ea86-408a-a6c1-545497badbbb",
        roles=["admin", "approver"],
        entity_id="ent_21661ac1-a2a8-4465-a6c0-64474ba8181d",
    )
    """

    id: EntityUserId

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class CounterpartyCustomizationAccount(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.entity_types import CounterpartyCustomizationAccount

    CounterpartyCustomizationAccount(
        account_id="85866843",
        postal_code="94105",
        name_on_account="John Doe",
    )
    """

    account_id: str = pydantic.Field(alias="accountId")
    """
    The ID the counterparty has assigned to this account.
    """

    postal_code: typing.Optional[str] = pydantic.Field(alias="postalCode", default=None)
    """
    The postal code the counterparty has assigned to this account.
    """

    name_on_account: typing.Optional[str] = pydantic.Field(alias="nameOnAccount", default=None)
    """
    The name on the account the counterparty has assigned to this account.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
